"
 COPYRIGHT (c) 1992 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

Object subclass:#ObjectMemory
       instanceVariableNames:''
       classVariableNames:'InternalErrorHandler UserInterruptHandler TimerInterruptHandler
                           SpyInterruptHandler StepInterruptHandler ExceptionInterruptHandler
                           ErrorInterruptHandler MemoryInterruptHandler SignalInterruptHandler
                           ChildSignalInterruptHandler DisposeInterruptHandler
                           RecursionInterruptHandler IOInterruptHandler
			   CustomInterruptHandler

                           AllocationFailureSignal
                           IncrementalGCLimit
                           Dependents
			   ImageName'
       poolDictionaries:''
       category:'System-Support'
!

ObjectMemory comment:'
COPYRIGHT (c) 1992 by Claus Gittinger
             All Rights Reserved
'!

!ObjectMemory class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1992 by Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

version
"
$Header: /cvs/stx/stx/libbasic/Attic/ObjMem.st,v 1.12 1994-06-02 16:21:03 claus Exp $
"
!

documentation
"
    This class contains access methods to the system memory -
    in previous versions this stuff used to be in the Smalltalk class.
    It has been separated for better overall structure.

    Many methods here are for debuging purposes only, and not standard.
    Do not depend on them being there - some may vanish ...
    (especially those, that depend on a specific GC implementation)

    kludge:
    The InterruptHandler variables are known by the runtime system -
    they are the objects that get an interrupt message when the event
    occurs.

    ClassVariables:

        InternalErrorHandler            gets informed, when some runtime error occurs
                                        (usually fatal)

        UserInterruptHandler            gets informed when CNTL-C is pressed
        TimerInterruptHandler           gets alarm timer interrupts
        SpyInterruptHandler             another alarm timer
        StepInterruptHandler            gets single step interrupts
        ExceptionInterruptHandler       gets floating point exceptions
        ErrorInterruptHandler           gets graphic device errors
        MemoryInterruptHandler          gets soon-out-of-memory conditions
        SignalInterruptHandler          gets unix signals
        ChildSignalInterruptHandler     gets child death signals
        DisposeInterruptHandler         gets informed, when an object is disposed from 
                                        a shadowArray
        RecursionInterruptHandler       gets recursion limit violations
        IOInterruptHandler              gets SIGIO unix signals
        CustomInterruptHandler          gets custom interrupts

        AllocationFailureSignal         signal raised when a new fails (see Behavior)
        IngrementalGCLimit              number of bytes, that must be allocated since
                                        last full garbage collect to turn on incremental
                                        collector.
        Dependents                      keep my dependents locally (its faster) for
                                        all those registries
"
! !

!ObjectMemory class methodsFor:'initialization'!

initialize
    AllocationFailureSignal isNil ifTrue:[
        AllocationFailureSignal := (Signal new) mayProceed:true.
        AllocationFailureSignal notifierString:'allocation failure'.
    ].
    IncrementalGCLimit := 500000.
    MemoryInterruptHandler := self
! !

!ObjectMemory class methodsFor:'signal access'!

allocationFailureSignal
    "return the signal raised when an object allocation failed"

    ^ AllocationFailureSignal
! !

!ObjectMemory class methodsFor:'dependents access'!

dependents
    "return the colleciton of my dependents"

    ^ Dependents
!

dependents:aCollection
    "set the dependents collection"

    Dependents := aCollection
! !

!ObjectMemory class methodsFor:'cache management'!

flushInlineCachesForClass:aClass
    "flush inlinecaches for calls to aClass"

%{  /* NOCONTEXT */
    __flushInlineCachesFor(aClass);
%}
!

flushInlineCachesWithArgs:nargs
    "flush inlinecaches for calls with nargs arguments"

%{  /* NOCONTEXT */
    __flushInlineCaches(_intVal(nargs));
%}
!

flushInlineCachesFor:aClass withArgs:nargs
    "flush inlinecaches for calls to aClass with nargs arguments"

%{  /* NOCONTEXT */
    __flushInlineCachesForAndNargs(aClass, _intVal(nargs));
%}
!

flushInlineCaches
    "flush all inlinecaches"

%{  /* NOCONTEXT */
    __flushAllInlineCaches();
%}
!

flushMethodCacheFor:aClass
    "flush the method cache for sends to aClass"

%{  /* NOCONTEXT */
    __flushMethodCacheFor(aClass);
%}
!

flushMethodCache
    "flush the method cache"

%{  /* NOCONTEXT */
    __flushMethodCache();
%}
!

flushCachesFor:aClass
    "flush method and inline caches for aClass"

%{  /* NOCONTEXT */
    __flushMethodCacheFor(aClass);
    __flushInlineCachesFor(aClass);
%}
!

flushCaches
    "flush method and inline caches for all classes"

%{  /* NOCONTEXT */
    __flushMethodCache();
    __flushAllInlineCaches();
%}
! !

!ObjectMemory class methodsFor:'enumeration'!

allObjectsDo:aBlock
    "evaluate the argument, aBlock for all objects in the system.
     There is one caveat: if a compressing oldSpace collect
     occurs while looping over the objects, the loop cannot be
     continued (for some internal reasons). In this case, false
     is returned."

    |work|

%{  /* NOREGISTER - work may not be placed into a register here */
    nonTenuringScavenge(__context);
    /*
     * allObjectsDo needs a temporary to hold newSpace objects
     */
    if (__allObjectsDo(&aBlock, &work COMMA_CON) < 0) {
	RETURN (false);
    }
%}.
    ^ true
!

allOldObjectsDo:aBlock
    "evaluate the argument, aBlock for all old objects in the system.
     For debugging and tests only - do not use"
%{
    if (__allObjectsDo(&aBlock, (OBJ *)0 COMMA_CON) < 0) {
        RETURN (false);
    }
%}. 
    ^ true
! !

!ObjectMemory class methodsFor:'handler access'!

internalErrorHandler
    "return the handler for ST/X internal errors.
     An internal error is reported for example when a methods
     bytecode is not a ByteArray, the selector table is not an Array
     etc.  Those should not occur in normal circumstances."

    ^ InternalErrorHandler
!

userInterruptHandler
    "return the handler for CNTL-C interrupt handling"

    ^ UserInterruptHandler
!

userInterruptHandler:aHandler
    "set the handler for CNTL-C interrupt handling"

    UserInterruptHandler := aHandler
!

timerInterruptHandler
    "return the handler for timer interrupts"

    ^ TimerInterruptHandler
!

timerInterruptHandler:aHandler
    "set the handler for timer interrupts"

    TimerInterruptHandler := aHandler
!

spyInterruptHandler
    "return the handler for spy-timer interrupts"

    ^ SpyInterruptHandler
!

spyInterruptHandler:aHandler
    "set the handler for spy-timer interrupts"

    SpyInterruptHandler := aHandler
!

stepInterruptHandler
    "return the handler for single step interrupts"

    ^ StepInterruptHandler
!

stepInterruptHandler:aHandler
    "set the handler for single step interrupts"

    StepInterruptHandler := aHandler
!

exceptionInterruptHandler
    "return the handler for floating point exception interrupts"

    ^ ExceptionInterruptHandler
!

errorInterruptHandler
    "return the handler for display error interrupts"

    ^ ErrorInterruptHandler
!

errorInterruptHandler:aHandler
    "set the handler for display error interrupts"

    ErrorInterruptHandler := aHandler
!

signalInterruptHandler
    "return the handler for UNIX-signal interrupts"

    ^ SignalInterruptHandler
!

signalInterruptHandler:aHandler
    "set the handler for UNIX-signal interrupts"

    SignalInterruptHandler := aHandler
!

childSignalInterruptHandler
    "return the handler for UNIX-death-of-a-childprocess-signal interrupts"

    ^ ChildSignalInterruptHandler
!

disposeInterruptHandler
    "return the handler for object disposal interrupts"

    ^ DisposeInterruptHandler
!

disposeInterruptHandler:aHandler
    "set the handler for object disposal interrupts"

    DisposeInterruptHandler := aHandler
!

recursionInterruptHandler
    "return the handler for recursion/stack overflow interrupts"

    ^ RecursionInterruptHandler
!

recursionInterruptHandler:aHandler
    "set the handler for recursion/stack overflow interrupts"

    RecursionInterruptHandler := aHandler
!

ioInterruptHandler
    "return the handler for I/O available signal interrupts (SIGIO/SIGPOLL)"

    ^ IOInterruptHandler
!

ioInterruptHandler:aHandler
    "set the handler for I/O available signal interrupts (SIGIO/SIGPOLL)"

    IOInterruptHandler := aHandler
!

customInterruptHandler
    "return the handler for custom interrupts"

    ^ CustomInterruptHandler
!

customInterruptHandler:aHandler
    "set the handler for custom interrupts"

    CustomInterruptHandler := aHandler
! !

!ObjectMemory class methodsFor:'queries'!

newSpaceSize
    "return the total size of the new space - this is usually fix"

%{  /* NOCONTEXT */
    RETURN ( _MKSMALLINT(__newSpaceSize()) );
%}
    "ObjectMemory newSpaceSize"
!

oldSpaceSize
    "return the total size of the old space. - may grow slowly"

%{  /* NOCONTEXT */
    RETURN ( _MKSMALLINT(__oldSpaceSize()) );
%}
    "ObjectMemory oldSpaceSize"
!

newSpaceUsed
    "return the number of bytes allocated for new objects.
     The returned value is usually obsolete as soon as you do
     something with it ..."

%{  /* NOCONTEXT */
    RETURN ( _MKSMALLINT(__newSpaceUsed()) );
%}
    "ObjectMemory newSpaceUsed"
!

oldSpaceUsed
    "return the number of bytes allocated for old objects.
     (This includes the free lists)"

%{  /* NOCONTEXT */
    RETURN ( _MKSMALLINT(__oldSpaceUsed()) );
%}
    "ObjectMemory oldSpaceUsed"
!

freeSpace
    "return the number of bytes in the free lists.
     (which is included in oldSpaceUsed)"

%{  /* NOCONTEXT */
    RETURN ( _MKSMALLINT(__freeSpace()) );
%}
    "ObjectMemory freeSpace"
!

bytesUsed
    "return the number of bytes allocated for objects -
     this number is not exact, since some objects may be dead"

%{  /* NOCONTEXT */
    RETURN ( _MKSMALLINT(__oldSpaceUsed() + __newSpaceUsed() - __freeSpace()) );
%}
    "ObjectMemory bytesUsed"
!

oldSpaceAllocatedSinceLastGC
    "return the number of bytes allocated for old objects since the
     last garbage collect occured"

%{  /* NOCONTEXT */
    RETURN ( _MKSMALLINT(__oldSpaceAllocatedSinceLastGC()) );
%}
    "ObjectMemory oldSpaceAllocatedSinceLastGC"
!

numberOfObjects
    "return the number of objects in the system."

    |tally|

    tally := 0.
    self allObjectsDo:[:obj | tally := tally + 1].
    ^ tally

    "ObjectMemory numberOfObjects"
!

printReferences:anObject
    "for debugging: print referents to anObject - will vanish.
     use 'whoReferences:anObject' below (or 'anObject allOwners')"

%{
    _printRefChain(__context, anObject);
%}
!

whoReferences:anObject
    "return a collection of objects referencing the argument, anObject"

    |aCollection|

    aCollection := IdentitySet new.
    self allObjectsDo:[:o |
        (o references:anObject) ifTrue:[
            aCollection add:o
        ]
    ].
    (aCollection size == 0) ifTrue:[
        "actually this cannot happen - there is always one"
        ^ nil
    ].
    ^ aCollection
!

addressOf:anObject
    "return the core address of anObject as an integer
     - since objects may move around, the returned value is invalid after the
     next scavenge/collect.
     Use only for debugging."

%{  /* NOCONTEXT */

    if (! _isNonNilObject(anObject)) {
        RETURN ( nil );
    }
    RETURN ( _MKSMALLINT( (int)anObject ) );
%}
    "
    |p|
    p := Point new.
    (ObjectMemory addressOf:p) printNL.
    ObjectMemory scavenge.
    (ObjectMemory addressOf:p) printNL.
    "
!

sizeOf:anObject
    "return the size of anObject in bytes.
     Use only for debugging/memory monitoring."

%{  /* NOCONTEXT */

    RETURN ( _isNonNilObject(anObject) ? _MKSMALLINT(_qSize(anObject)) : _MKSMALLINT(0) )
%}
    "
     |hist big nw|

     hist := Array new:100 withAll:0.
     big := 0.
     ObjectMemory allObjectsDo:[:o |
         nw := (ObjectMemory sizeOf:o) // 4 + 1.
         nw > 100 ifTrue:[
            big := big + 1
         ] ifFalse:[
            hist at:nw put:(hist at:nw) + 1
         ].
     ].
     hist printNL.
     big printNL
    "
!

spaceOf:anObject
    "return the memory space, in which anObject is.
     - since objects may move between spaces, returned value is invalid after the
     next scavenge/collect.
     Use only for debugging."

%{  /* NOCONTEXT */

    if (! _isNonNilObject(anObject)) {
        RETURN ( nil );
    }
    RETURN ( _MKSMALLINT( _qSpace(anObject) ) );
%}
    "
    |p|
    p := Point new.
    (ObjectMemory spaceOf:p) printNL.
    1 to:100 do:[:i |
        ObjectMemory scavenge.
    ].
    (ObjectMemory spaceOf:p) printNL.
    1 to:100 do:[:i |
        ObjectMemory tenuringScavenge.
    ].
    (ObjectMemory spaceOf:p) printNL.
    "
!

ageOf:anObject
    "return the number of scavenges, an object has survived
     in new space. For old objects, the returned number is invalid.
     Use only for debugging."

%{  /* NOCONTEXT */

    if (! _isNonNilObject(anObject)) {
        RETURN ( 0 );
    }
    RETURN ( _MKSMALLINT( _GET_AGE(anObject) ) );
%}
    "
    |p|
    p := Point new.
    (ObjectMemory ageOf:p) printNL.
    ObjectMemory tenuringScavenge.
    (ObjectMemory spaceOf:p) printNL.
    ObjectMemory tenuringScavenge.
    (ObjectMemory spaceOf:p) printNL.
    ObjectMemory tenuringScavenge.
    (ObjectMemory spaceOf:p) printNL.
    ObjectMemory tenuringScavenge.
    (ObjectMemory spaceOf:p) printNL.
    "
! !

!ObjectMemory class methodsFor:'garbage collector control'!

garbageCollect
    "search for and free garbage in the oldSpace
     (newSpace is cleaned automatically)
     - can take a long time if paging is involved
     - when no paging is involved, its faster than I thought :-)"
%{
    __garbageCollect(__context);
%}

    "ObjectMemory garbageCollect"
!

moreOldSpace:howMuch
    "allocate howMuch bytes more for old objects.
     This is done automatically, when running out of space.
     This (currently) implies a compressing garbage collect - so its slow.
     May vanish."

%{  
    if (_isSmallInteger(howMuch))
        _moreOldSpace(__context, _intVal(howMuch));
%}
    "ObjectMemory moreOldSpace:1000000"
!

scavenge
    "collect young objects, without aging.
     Can be used to quickly get rid of shortly before allocated
     stuff. Is relatively fast (compared to oldspace collect)"
%{
    nonTenuringScavenge(__context);
%}

    "ObjectMemory scavenge"
!

tenuringScavenge
    "collect newspace stuff, with aging.
     For debugging only. This method may vanish.
     Is relatively fast (compared to oldspace collect)"
%{
    scavenge(__context);
%}

    "ObjectMemory tenuringScavenge"
!

tenure
    "force all new stuff into old-space - effectively making
     all living young objects become old objects.
     Is relatively fast (compared to oldspace collect).
     For debugging only. This method may vanish."
%{
    tenure(__context);
%}

    "ObjectMemory tenure"
!

markAndSweep
    "mark/sweep garbage collector; perform a
     full mark&sweep collect.
     Warning: this may take some time."
%{
    markAndSweep(__context);
%}

    "ObjectMemory markAndSweep"
!

gcStep
    "one incremental garbage collect step.
     Mark or sweep some small number of objects. This
     method will return after a reasonable (short) time.
     This is used by the ProcessorScheduler at idle times."
%{
    incrGCstep(__context);
%}
!

incrementalGCLimit:aNumber
    "set the limit for incremental GC activation.
     This is used by the ProcessorScheduler."

    IncrementalGCLimit := aNumber

    "
     ObjectMemory incrementalGCLimit:100000
    "
!

incrementalGCLimit
    "return the limit for incremental GC activation.
     This is used by the ProcessorScheduler."

    ^ IncrementalGCLimit

    "
     ObjectMemory incrementalGCLimit
    "
!

turnGarbageCollectorOff
    "turn off garbage collector by forcing new objects to be
     allocated in oldSpace (instead of newSpace)
     this method is somewhat dangerous: if collector is turned off,
     and too many objects are created, the system may run into trouble
     (i.e. oldSpace becomes full) and be forced to perform a full mark&sweep
     or even a compressing collect.
     Use this only for measurement purposes or when realtime behavior
     is required for a limited time period. No warranty."
%{
    allocForceSpace(OLDSPACE);
%}
!

turnGarbageCollectorOn
    "turn garbage collector on again"

%{
    allocForceSpace(9999);
%}
! !

!ObjectMemory class methodsFor:'physical memory access'!

makeOld:anObject
    "move anObject into oldSpace - for debugging only;
     may vanish. Dont use it."
%{
    if (__moveToOldSpace(anObject, __context) < 0) {
	RETURN (false);
    }
%}.
    ^ true
!

oldSpacePagesDo:aBlock
    "evaluates aBlock for all pages in the oldSpace, passing
     the pages address as argument. 
     For internal use only."
%{
    if (__oldSpacePagesDo(&aBlock COMMA_CON) < 0) {
        RETURN (false);
    }
%}.
    ^ true
!

pageIsInCore:aPageNumber
    "return true, if the page (as enumerated via oldSpacePagesDo:)
     is in memory; false, if currently paged out. For internal
     use / monitors only; may vanish."

%{
#ifdef HAS_MINCORE
    int pageSize = getpagesize();
    char result[10];
    INT addr;

    if (_isSmallInteger(aPageNumber)) {
	addr = _intVal(aPageNumber) & ~(pageSize - 1);
    } else {
	addr = ((INT)aPageNumber) & ~(pageSize - 1);
    }
    if (mincore(addr, pageSize-1, result) < 0) {
	RETURN (true);
    }
    RETURN ((result[0] & 1) ? true : false);
#endif
%}.
    "OS does not supply this info - assume yes"
    ^ true
! !

!ObjectMemory class methodsFor:'low memory handling'!

memoryInterrupt
    "when a low-memory condition arises, ask all classes to
     remove possibly cached data - this may help a bit"

    Smalltalk allClasses do:[:aClass |
        aClass lowSpaceCleanup
    ].

    self error:'almost out of memory'
! !

!ObjectMemory class methodsFor:'system management'!

loadClassBinary:aClassName
    "find the object file for aClassName and -if found - load it;
     this one loads precompiled object files"

    |fName newClass upd|

    fName := self fileNameForClass:aClassName.
    fName notNil ifTrue:[
        upd := Class updateChanges:false.
        [
            self loadBinary:(fName , '.o')
        ] valueNowOrOnUnwindDo:[
            Class updateChanges:upd
        ].
        newClass := self at:(aClassName asSymbol).
        (newClass notNil and:[newClass implements:#initialize]) ifTrue:[
            newClass initialize
        ]
    ]
!

imageName
    "return the filename of the current image, or nil
     if not running from an image."

    ^ ImageName
!

nameForSnapshot
    "return the filename of the current image or,
     if not running from an image, the default name 'st.img'"

    (ImageName isNil or:[ImageName isBlank]) ifTrue:[
        ^ 'st.img'
    ].
    ^ ImageName
!

snapShot
    "create a snapshot"

    ImageName isNil ifTrue:[
        ImageName := 'st.img'
    ].
    self snapShotOn:(self nameForSnapshot)

    "ObjectMemory snapShot"
!

snapShotOn:aFileName
    "create a snapshot in the given file"

    |ok|

    "give others a chance to fix things"
    self changed:#save.

%{  /* STACK:32000 */

    OBJ __snapShotOn();
    OBJ retVal;

    if (__isString(aFileName)) {
        BLOCKINTERRUPTS();
        ok = __snapShotOn(__context, _stringVal(aFileName));
        UNBLOCKINTERRUPTS();
    }
%}
.
    ok ifTrue:[
        Class addChangeRecordForSnapshot:aFileName
    ].
    ^ ok

    "ObjectMemory snapShotOn:'myimage.img'"
!

applicationImageOn:aFileName for:startupClass selector:startupSelector
    "create a snapshot which will come up without any views 
     but starts up an application by sending startupClass the startupSelector"

    |viewsKnown savedIdleBlocks savedTimeoutBlocks savedTranscript
     savedRoot|

    viewsKnown := Display knownViews.
    savedIdleBlocks := Display idleBlocks.
    savedTimeoutBlocks := Display timeOutBlocks.
    savedTranscript := Transcript.
    savedRoot := RootView.

    "a kludge: save image with modified knownViews, no idle- and timeoutblocks
     and also Transcript set to StdErr ..."

    Display knownViews:nil.
    Display idleBlocks:nil.
    Display timeOutBlocks:nil.
    RootView := nil.

    Transcript := Stderr.
    Smalltalk startupClass:startupClass selector:startupSelector arguments:nil.
    self snapShotOn:aFileName.
    Smalltalk startupClass:nil selector:nil arguments:nil.

    RootView := savedRoot.
    Transcript := savedTranscript.
    Display knownViews:viewsKnown.
    Display idleBlocks:savedIdleBlocks.
    Display timeOutBlocks:savedTimeoutBlocks

    "ObjectMemory applicationImageOn:'draw.img' for:DrawTool selector:#start"
    "ObjectMemory applicationImageOn:'pm.img' for:PMSimulator selector:#start"
!

minimumApplicationImageOn:aFileName for:startupClass selector:startupSelector
    "create a snapshot which will come up without any views 
     but starts up an application by sending startupClass the startupSelector.
     All unneeded info is stripped from the saved image."

    "create a temporary image, for continuation"
    self snapShotOn:'temp.img'.

    Display knownViews do:[:aView |
        aView notNil ifTrue:[
            aView superView isNil ifTrue:[
                aView destroy
            ]
        ]
    ].

    self stripImage.

    self applicationImageOn:aFileName for:startupClass selector:startupSelector.

    "continue in old image"

    OperatingSystem exec:(Arguments at:1)
           withArguments:#('smalltalk' '-i' 'temp.img') , (Arguments copyFrom:2)

    "ObjectMemory minimumApplicationImageOn:'clock1.img' for:Clock selector:#start"
    "ObjectMemory applicationImageOn:'clock2.img' for:Clock selector:#start"
!

stripImage
    "remove all unneeded stuff from the image - much more is possible here"

    "remove all class comments & source"

    Smalltalk allBehaviorsDo:[:aClass |
        aClass setComment:nil.
        aClass methodArray do:[:aMethod |
            aMethod source:''.
            aMethod category:#none 
        ]
    ].
    self garbageCollect
! !
